@@ -37,22 +37,31 @@ def test(hurl_file):
     options_file = hurl_file.replace('.hurl','.options')
     curl_file = hurl_file.replace('.hurl','.curl')
     json_output_file = hurl_file.replace('.hurl','.output.json')
+    profile_file = hurl_file.replace('.hurl','.profile')
 
     options = []
     if os.path.exists(options_file):
          options = open(options_file).read().strip().split(' ')
     if os.path.exists(curl_file):
         options.append('--verbose')
-        
-    if os.path.exists(json_output_file):
-        json_output_actual_file =  os.path.join(tempfile.mkdtemp(), 'output.json')
-        options.append('--json')
-        options.append(json_output_actual_file)
 
+    if os.path.exists(json_output_file):
+        options.append('--json')
+
+    env = os.environ.copy()
+    if os.path.exists(profile_file):
+        for line in open(profile_file).readlines():
+            line = line.strip()
+            if line == '':
+                continue
+            index = line.index('=')
+            name = line[:index]
+            value = line[(index+1):]
+            env[name] = value
 
     cmd = ['hurl', hurl_file] + options
     print(' '.join(cmd))
-    result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+    result = subprocess.run(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE,env=env)
 
     # exit code 
     f = hurl_file.replace('.hurl','.exit')
@@ -75,6 +84,12 @@ def test(hurl_file):
              print(f'actual: <{actual}>\nexpected: <{expected}>')
              sys.exit(1)
 
+    # stdout (json)
+    if os.path.exists(json_output_file):
+        expected = open(json_output_file).read()
+        actual = result.stdout
+        check_json_output.check(expected, actual)
+
     # stderr
     f = hurl_file.replace('.hurl', '.' + get_os() + '.err')
     if os.path.exists(f):
@@ -119,9 +134,6 @@ def test(hurl_file):
                 print('actual:   %s' % actual_commands[i])
                 sys.exit(1)
 
-    if os.path.exists(json_output_file):
-         check_json_output.check_file(json_output_file, json_output_actual_file)
-
 
 def main():
     for hurl_file in sys.argv[1:]:
