@@ -10,7 +10,7 @@ def variables():
     assert request.headers['Age'] == '30'
     assert request.headers['Female'] == 'true'
     assert request.headers['Id'] == '123'
-    assert request.headers['Height'] == '1.700000000000000000'
+    assert request.headers['Height'] == '1.7'
     assert request.headers['A_null'] == 'null'
 
     s = request.data.decode("utf-8")
@@ -20,7 +20,7 @@ def variables():
     assert data['female'] == True
     assert data['id'] == '123'
     assert data['height'] == 1.7
-    assert data['a_null'] == None
+    assert data['a_null'] is None
     return ''
 
 
