@@ -25,7 +25,7 @@ use atty::Stream;
 use colored::*;
 
 use hurl::cli;
-use hurl::cli::{CliError, CliOptions};
+use hurl::cli::{CliError, CliOptions, OutputType};
 use hurl::http;
 use hurl::report;
 use hurl::runner;
@@ -52,6 +52,29 @@ pub fn init_colored() {
     colored::control::set_virtual_terminal(true);
 }
 
+#[cfg(target_family = "unix")]
+pub fn write_bytes(buf: &[u8]) -> Result<(), CliError> {
+    let stdout = io::stdout();
+    let mut handle = stdout.lock();
+    handle.write_all(buf).map_err(|_| CliError {
+        message: "Error writing output".to_string(),
+    })
+}
+
+#[cfg(target_family = "windows")]
+pub fn write_bytes(buf: &[u8]) -> Result<(), CliError> {
+    if atty::is(Stream::Stdout) {
+        println!("{}", String::from_utf8_lossy(buf));
+        Ok(())
+    } else {
+        let stdout = io::stdout();
+        let mut handle = stdout.lock();
+        handle.write_all(buf).map_err(|_| CliError {
+            message: "Error writing output".to_string(),
+        })
+    }
+}
+
 #[derive(Clone, Debug, PartialEq, Eq)]
 struct Progress {
     pub current: usize,
@@ -79,7 +102,7 @@ fn execute(
     };
 
     if let Some(Progress { current, total }) = progress {
-        eprintln!("{}: running [{}/{}]", filename, current + 1, total);
+        eprintln!("{}: RUNNING [{}/{}]", filename, current + 1, total);
     }
     let log_parser_error =
         cli::make_logger_parser_error(lines.clone(), cli_options.color, optional_filename.clone());
@@ -124,6 +147,7 @@ fn execute(
                 }
             }
 
+            let cacert_file = cli_options.cacert_file;
             let follow_location = cli_options.follow_location;
             let verbose = cli_options.verbose;
             let insecure = cli_options.insecure;
@@ -149,6 +173,7 @@ fn execute(
                 Some(filename) => filename,
             };
             let options = http::ClientOptions {
+                cacert_file,
                 follow_location,
                 max_redirect,
                 cookie_input_file,
@@ -180,6 +205,7 @@ fn execute(
                 variables: cli_options.variables,
                 to_entry: cli_options.to_entry,
                 context_dir,
+                ignore_asserts: cli_options.ignore_asserts,
                 pre_entry,
                 post_entry,
             };
@@ -187,17 +213,17 @@ fn execute(
                 hurl_file,
                 &mut client,
                 filename.to_string(),
-                options,
+                &options,
                 &log_verbose,
                 &log_error_message,
                 &log_runner_error,
             );
             if cli_options.progress {
                 let status = match (result.success, cli_options.color) {
-                    (true, true) => "success".green().to_string(),
-                    (true, false) => "success".to_string(),
-                    (false, true) => "failure".red().to_string(),
-                    (false, false) => "failure".to_string(),
+                    (true, true) => "SUCCESS".green().to_string(),
+                    (true, false) => "SUCCESS".to_string(),
+                    (false, true) => "FAILURE".red().to_string(),
+                    (false, false) => "FAILURE".to_string(),
                 };
                 eprintln!("{}: {}", filename, status);
             }
@@ -220,13 +246,30 @@ pub fn unwrap_or_exit<T>(
 }
 
 fn main() {
-    let app = cli::app();
+    let version_info = format!(
+        "{} {}",
+        clap::crate_version!(),
+        http::libcurl_version_info().join(" ")
+    );
+    let app = cli::app().version(version_info.as_str());
     let matches = app.clone().get_matches();
     init_colored();
-    let mut filenames = match matches.values_of("INPUT") {
-        None => vec![],
-        Some(v) => v.collect(),
+
+    let verbose = matches.is_present("verbose") || matches.is_present("interactive");
+    let log_verbose = cli::make_logger_verbose(verbose);
+    let color = cli::output_color(matches.clone());
+    let log_error_message = cli::make_logger_error_message(color);
+    let cli_options = unwrap_or_exit(&log_error_message, cli::parse_options(matches.clone()));
+
+    let mut filenames = vec![];
+    if let Some(values) = matches.values_of("INPUT") {
+        for value in values {
+            filenames.push(value.to_string());
+        }
     };
+    for filename in &cli_options.glob_files {
+        filenames.push(filename.to_string());
+    }
 
     if filenames.is_empty() && atty::is(Stream::Stdin) {
         if app.clone().print_help().is_err() {
@@ -235,18 +278,12 @@ fn main() {
         println!();
         std::process::exit(EXIT_ERROR_COMMANDLINE);
     } else if filenames.is_empty() {
-        filenames.push("-");
+        filenames.push("-".to_string());
     }
 
     let current_dir_buf = std::env::current_dir().unwrap();
     let current_dir = current_dir_buf.as_path();
 
-    let verbose = matches.is_present("verbose") || matches.is_present("interactive");
-    let log_verbose = cli::make_logger_verbose(verbose);
-    let color = cli::output_color(matches.clone());
-    let log_error_message = cli::make_logger_error_message(color);
-    let cli_options = unwrap_or_exit(&log_error_message, cli::parse_options(matches.clone()));
-
     let mut hurl_results = vec![];
 
     let cookies_output_file = match cli_options.cookie_output_file.clone() {
@@ -261,6 +298,20 @@ fn main() {
     };
 
     let start = Instant::now();
+
+    let doc = if let Ok(doc) = report::create_or_get_junit_report(cli_options.junit_file.clone()) {
+        doc
+    } else {
+        log_error_message(false, "Error creating Junit XML report");
+        std::process::exit(EXIT_ERROR_UNDEFINED);
+    };
+    let mut testsuite = if let Ok(doc) = report::add_testsuite(&doc) {
+        doc
+    } else {
+        log_error_message(false, "Error creating Junit XML report");
+        std::process::exit(EXIT_ERROR_UNDEFINED);
+    };
+
     for (current, filename) in filenames.iter().enumerate() {
         let contents = match cli::read_to_string(filename) {
             Ok(v) => v,
@@ -280,15 +331,19 @@ fn main() {
         };
         let hurl_result = execute(
             filename,
-            contents,
+            contents.clone(),
             current_dir,
             cli_options.clone(),
             &log_verbose,
             &log_error_message,
             progress,
         );
+        hurl_results.push(hurl_result.clone());
 
-        if hurl_result.errors().is_empty() && !cli_options.interactive {
+        if matches!(cli_options.output_type, OutputType::ResponseBody)
+            && hurl_result.errors().is_empty()
+            && !cli_options.interactive
+        {
             // default
             // last entry + response + body
             if let Some(entry_result) = hurl_result.entries.last() {
@@ -339,7 +394,7 @@ fn main() {
                     cli::log_info("no response has been received");
                 }
             } else {
-                let source = if *filename == "-" {
+                let source = if filename.as_str() == "-" {
                     "".to_string()
                 } else {
                     format!("for file {}", filename).to_string()
@@ -351,16 +406,33 @@ fn main() {
             };
         }
 
-        hurl_results.push(hurl_result.clone());
+        let lines: Vec<String> = regex::Regex::new(r"\n|\r\n")
+            .unwrap()
+            .split(&contents)
+            .map(|l| l.to_string())
+            .collect();
+        if matches!(cli_options.output_type, OutputType::Json) {
+            let json_result = hurl_result.to_json(&lines);
+            let serialized = serde_json::to_string(&json_result).unwrap();
+            let s = format!("{}\n", serialized);
+            unwrap_or_exit(
+                &log_error_message,
+                write_output(s.into_bytes(), cli_options.output.clone()),
+            );
+        }
+        if cli_options.junit_file.is_some() {
+            unwrap_or_exit(
+                &log_error_message,
+                report::add_testcase(&doc, &mut testsuite, hurl_result, &lines),
+            );
+        }
     }
-    let duration = start.elapsed().as_millis();
 
-    if let Some(file_path) = cli_options.json_file {
-        log_verbose(format!("Writing json report to {}", file_path.display()).as_str());
-        unwrap_or_exit(
-            &log_error_message,
-            report::write_json_report(file_path, hurl_results.clone()),
-        );
+    if let Some(file_path) = cli_options.junit_file.clone() {
+        log_verbose(format!("Writing Junit report to {}", file_path.display()).as_str());
+        if doc.save_file(&file_path.to_string_lossy()).is_err() {
+            log_error_message(false, format!("Failed to save to {:?}", file_path).as_str());
+        }
     }
 
     if let Some(dir_path) = cli_options.html_dir {
@@ -371,7 +443,10 @@ fn main() {
         );
 
         for filename in filenames {
-            unwrap_or_exit(&log_error_message, format_html(filename, dir_path.clone()));
+            unwrap_or_exit(
+                &log_error_message,
+                format_html(filename.as_str(), dir_path.clone()),
+            );
         }
     }
 
@@ -384,7 +459,9 @@ fn main() {
     }
 
     if cli_options.summary {
-        print_summary(duration, hurl_results.clone())
+        let duration = start.elapsed().as_millis();
+        let summary = get_summary(duration, hurl_results.clone());
+        eprintln!("{}", summary.as_str());
     }
 
     std::process::exit(exit_code(hurl_results));
@@ -438,15 +515,7 @@ fn format_html(input_file: &str, dir_path: PathBuf) -> Result<(), CliError> {
 
 fn write_output(bytes: Vec<u8>, filename: Option<String>) -> Result<(), CliError> {
     match filename {
-        None => {
-            let stdout = io::stdout();
-            let mut handle = stdout.lock();
-
-            handle
-                .write_all(bytes.as_slice())
-                .expect("writing bytes to console");
-            Ok(())
-        }
+        None => write_bytes(bytes.as_slice()),
         Some(filename) => {
             let path = Path::new(filename.as_str());
             let mut file = match std::fs::File::create(&path) {
@@ -511,21 +580,30 @@ fn write_cookies_file(file_path: PathBuf, hurl_results: Vec<HurlResult>) -> Resu
     Ok(())
 }
 
-fn print_summary(duration: u128, hurl_results: Vec<HurlResult>) {
+fn get_summary(duration: u128, hurl_results: Vec<HurlResult>) -> String {
     let total = hurl_results.len();
     let success = hurl_results.iter().filter(|r| r.success).count();
     let failed = total - success;
-    eprintln!("--------------------------------------------------------------------------------");
-    eprintln!("executed: {}", total);
-    eprintln!(
-        "success: {} ({:.1}%)",
-        success,
-        100.0 * success as f32 / total as f32
+    let mut s =
+        "--------------------------------------------------------------------------------\n"
+            .to_string();
+    s.push_str(format!("Executed:  {}\n", total).as_str());
+    s.push_str(
+        format!(
+            "Succeeded: {} ({:.1}%)\n",
+            success,
+            100.0 * success as f32 / total as f32
+        )
+        .as_str(),
     );
-    eprintln!(
-        "failed: {} ({:.1}%)",
-        failed,
-        100.0 * failed as f32 / total as f32
+    s.push_str(
+        format!(
+            "Failed:    {} ({:.1}%)\n",
+            failed,
+            100.0 * failed as f32 / total as f32
+        )
+        .as_str(),
     );
-    eprintln!("execution time: {}ms", duration);
+    s.push_str(format!("Duration:  {}ms\n", duration).as_str());
+    s
 }
