@@ -115,22 +115,14 @@ pub fn run(
         Ok(calls) => calls,
         Err(http_error) => {
             let runner_error = match http_error {
-                HttpError::CouldNotResolveProxyName => RunnerError::CouldNotResolveProxyName,
-                HttpError::CouldNotResolveHost(host) => RunnerError::CouldNotResolveHost(host),
-                HttpError::FailToConnect => RunnerError::FailToConnect,
-                HttpError::Timeout => RunnerError::Timeout,
                 HttpError::TooManyRedirect => RunnerError::TooManyRedirect,
                 HttpError::CouldNotParseResponse => RunnerError::CouldNotParseResponse,
-                HttpError::SslCertificate(description) => RunnerError::SslCertificate(
-                    description.unwrap_or_else(|| "SSL certificate problem".to_string()),
-                ),
-                HttpError::InvalidUrl => RunnerError::InvalidUrl(http_request.url.clone()),
                 HttpError::StatuslineIsMissing => RunnerError::HttpConnection {
                     message: "status line is missing".to_string(),
                     url: http_request.url.clone(),
                 },
-                HttpError::Other { description, .. } => RunnerError::HttpConnection {
-                    message: description,
+                HttpError::Libcurl { code, description } => RunnerError::HttpConnection {
+                    message: format!("({}) {}", code, description),
                     url: http_request.url.clone(),
                 },
             };
