@@ -15,14 +15,16 @@
  * limitations under the License.
  *
  */
+use crate::ast::*;
+
 use super::base64;
 use super::combinators::*;
 use super::error::*;
+use super::filename;
 use super::reader::Reader;
 use super::string::*;
 use super::template;
 use super::ParseResult;
-use crate::ast::*;
 
 pub fn space(reader: &mut Reader) -> ParseResult<'static, Whitespace> {
     let start = reader.state.clone();
@@ -96,7 +98,7 @@ pub fn zero_or_more_spaces<'a>(reader: &mut Reader) -> ParseResult<'a, Whitespac
 pub fn line_terminator(reader: &mut Reader) -> ParseResult<'static, LineTerminator> {
     // let start = p.state.clone();
     let space0 = zero_or_more_spaces(reader)?;
-    let comment = optional(|p1| comment(p1), reader)?;
+    let comment = optional(comment, reader)?;
     let nl = if reader.is_eof() {
         Whitespace {
             value: "".to_string(),
@@ -130,7 +132,7 @@ pub fn line_terminator(reader: &mut Reader) -> ParseResult<'static, LineTerminat
 }
 
 pub fn optional_line_terminators(reader: &mut Reader) -> ParseResult<'static, Vec<LineTerminator>> {
-    zero_or_more(|p2| recover(|p1| line_terminator(p1), p2), reader)
+    zero_or_more(|p2| recover(line_terminator, p2), reader)
 }
 
 pub fn comment(reader: &mut Reader) -> ParseResult<'static, Comment> {
@@ -330,38 +332,6 @@ pub fn hex(reader: &mut Reader) -> ParseResult<'static, Hex> {
     })
 }
 
-pub fn filename(reader: &mut Reader) -> ParseResult<'static, Filename> {
-    // this is an absolute file
-    // that you have to write with a relative name
-    // default root_dir is the hurl directory
-    let start = reader.state.clone();
-    let s = reader.read_while_escaping(|c| {
-        c.is_alphanumeric() || *c == '.' || *c == '/' || *c == '_' || *c == '-'
-    });
-    if s.is_empty() {
-        return Err(Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: ParseError::Filename {},
-        });
-    }
-    if s.starts_with('/') {
-        return Err(Error {
-            pos: start.pos,
-            recoverable: false,
-            inner: ParseError::Filename {},
-        });
-    }
-
-    Ok(Filename {
-        value: s,
-        source_info: SourceInfo {
-            start: start.pos,
-            end: reader.state.clone().pos,
-        },
-    })
-}
-
 pub fn null(reader: &mut Reader) -> ParseResult<'static, ()> {
     try_literal("null", reader)
 }
@@ -434,8 +404,12 @@ pub fn integer(reader: &mut Reader) -> ParseResult<'static, i64> {
 pub fn float(reader: &mut Reader) -> ParseResult<'static, Float> {
     // non recoverable after the dot
     // an integer is parsed ok as float => no like a computer language
-    let int = integer(reader)?;
-
+    let start = reader.state.cursor;
+    let sign = match try_literal("-", reader) {
+        Err(_) => "",
+        Ok(_) => "-",
+    };
+    let nat = natural(reader)?;
     try_literal(".", reader)?;
 
     if reader.is_eof() {
@@ -458,13 +432,9 @@ pub fn float(reader: &mut Reader) -> ParseResult<'static, Float> {
             },
         });
     }
-    let decimal = format!("{:0<18}", s).parse().unwrap();
-    let decimal_digits = s.len();
-    Ok(Float {
-        int,
-        decimal,
-        decimal_digits,
-    })
+    let value = format!("{}{}.{}", sign, nat, s).parse().unwrap();
+    let encoded = reader.from(start);
+    Ok(Float { value, encoded })
 }
 
 pub fn raw_string(reader: &mut Reader) -> ParseResult<'static, RawString> {
@@ -526,7 +496,7 @@ pub(crate) fn file(reader: &mut Reader) -> ParseResult<'static, File> {
     try_literal("file", reader)?;
     literal(",", reader)?;
     let space0 = zero_or_more_spaces(reader)?;
-    let f = filename(reader)?;
+    let f = filename::parse(reader)?;
     let space1 = zero_or_more_spaces(reader)?;
     literal(";", reader)?;
     Ok(File {
@@ -842,43 +812,6 @@ mod tests {
         );
     }
 
-    #[test]
-    fn test_filename() {
-        let mut reader = Reader::init("data/data.bin");
-        assert_eq!(
-            filename(&mut reader).unwrap(),
-            Filename {
-                value: String::from("data/data.bin"),
-                source_info: SourceInfo::init(1, 1, 1, 14),
-            }
-        );
-        assert_eq!(reader.state.cursor, 13);
-
-        let mut reader = Reader::init("data.bin");
-        assert_eq!(
-            filename(&mut reader).unwrap(),
-            Filename {
-                value: String::from("data.bin"),
-                source_info: SourceInfo::init(1, 1, 1, 9),
-            }
-        );
-        assert_eq!(reader.state.cursor, 8);
-    }
-
-    #[test]
-    fn test_filename_error() {
-        let mut reader = Reader::init("???");
-        let error = filename(&mut reader).err().unwrap();
-        assert_eq!(error.inner, ParseError::Filename {});
-        assert_eq!(error.pos, Pos { line: 1, column: 1 });
-
-        // can not absolute
-        let mut reader = Reader::init("/tmp/data.bin");
-        let error = filename(&mut reader).err().unwrap();
-        assert_eq!(error.inner, ParseError::Filename {});
-        assert_eq!(error.pos, Pos { line: 1, column: 1 });
-    }
-
     #[test]
     fn test_boolean() {
         let mut reader = Reader::init("true");
@@ -985,9 +918,8 @@ mod tests {
         assert_eq!(
             float(&mut reader).unwrap(),
             Float {
-                int: 1,
-                decimal: 0,
-                decimal_digits: 1,
+                value: 1.0,
+                encoded: "1.0".to_string()
             }
         );
         assert_eq!(reader.state.cursor, 3);
@@ -996,9 +928,8 @@ mod tests {
         assert_eq!(
             float(&mut reader).unwrap(),
             Float {
-                int: -1,
-                decimal: 0,
-                decimal_digits: 1,
+                value: -1.0,
+                encoded: "-1.0".to_string()
             }
         );
         assert_eq!(reader.state.cursor, 4);
@@ -1007,9 +938,8 @@ mod tests {
         assert_eq!(
             float(&mut reader).unwrap(),
             Float {
-                int: 1,
-                decimal: 100_000_000_000_000_000,
-                decimal_digits: 1,
+                value: 1.1,
+                encoded: "1.1".to_string()
             }
         );
         assert_eq!(reader.state.cursor, 3);
@@ -1018,9 +948,8 @@ mod tests {
         assert_eq!(
             float(&mut reader).unwrap(),
             Float {
-                int: 1,
-                decimal: 100_000_000_000_000_000,
-                decimal_digits: 3,
+                value: 1.1,
+                encoded: "1.100".to_string()
             }
         );
         assert_eq!(reader.state.cursor, 5);
@@ -1029,9 +958,8 @@ mod tests {
         assert_eq!(
             float(&mut reader).unwrap(),
             Float {
-                int: 1,
-                decimal: 10_000_000_000_000_000,
-                decimal_digits: 2,
+                value: 1.01,
+                encoded: "1.01".to_string()
             }
         );
         assert_eq!(reader.state.cursor, 4);
@@ -1040,23 +968,22 @@ mod tests {
         assert_eq!(
             float(&mut reader).unwrap(),
             Float {
-                int: 1,
-                decimal: 10_000_000_000_000_000,
-                decimal_digits: 3,
+                value: 1.01,
+                encoded: "1.010".to_string()
             }
         );
         assert_eq!(reader.state.cursor, 5);
 
-        let mut reader = Reader::init("-0.333333333333333333");
+        // provide more digits than necessary
+        let mut reader = Reader::init("-0.3333333333333333333");
         assert_eq!(
             float(&mut reader).unwrap(),
             Float {
-                int: 0,
-                decimal: 333_333_333_333_333_333,
-                decimal_digits: 18,
+                value: -0.3333333333333333,
+                encoded: "-0.3333333333333333333".to_string()
             }
         );
-        assert_eq!(reader.state.cursor, 21);
+        assert_eq!(reader.state.cursor, 22);
     }
 
     #[test]
@@ -1368,13 +1295,13 @@ mod tests {
             Hex {
                 space0: Whitespace {
                     value: " ".to_string(),
-                    source_info: SourceInfo::init(1, 5, 1, 6)
+                    source_info: SourceInfo::init(1, 5, 1, 6),
                 },
                 value: vec![255],
                 encoded: "ff".to_string(),
                 space1: Whitespace {
                     value: "".to_string(),
-                    source_info: SourceInfo::init(1, 8, 1, 8)
+                    source_info: SourceInfo::init(1, 8, 1, 8),
                 },
             }
         );
@@ -1385,13 +1312,13 @@ mod tests {
             Hex {
                 space0: Whitespace {
                     value: "".to_string(),
-                    source_info: SourceInfo::init(1, 5, 1, 5)
+                    source_info: SourceInfo::init(1, 5, 1, 5),
                 },
                 value: vec![1, 2, 3],
                 encoded: "010203".to_string(),
                 space1: Whitespace {
                     value: " ".to_string(),
-                    source_info: SourceInfo::init(1, 11, 1, 12)
+                    source_info: SourceInfo::init(1, 11, 1, 12),
                 },
             }
         );
